from colorama import init, Fore, Style
init()
from os import get_terminal_size
from datetime import datetime

from prompt_toolkit                                   import print_formatted_text
from prompt_toolkit.shortcuts                         import ProgressBar
from prompt_toolkit.formatted_text                    import HTML
from prompt_toolkit.shortcuts.progress_bar.formatters import *
from prompt_toolkit.styles                            import Style as pStyle
from prompt_toolkit.output.color_depth                import ColorDepth
from time import sleep

from xml.sax.saxutils                                 import escape

def _testinit(runu, interpreter, perimodu, default, context, types, exceptions):
    global run
    run = runu.run
    global RTResult
    RTResult = interpreter.RTResult
    global perimod
    perimod = perimodu
    global defaultvariables
    defaultvariables = default.defaultvariables
    global SymbolTable, Context
    SymbolTable = context.SymbolTable
    Context = context.Context
    global TypeObj
    TypeObj = types.TypeObj
    global Exc_BaseException, Exc_ReturnError
    Exc_BaseException = exceptions.Exc_BaseException
    Exc_ReturnError = exceptions.Exc_ReturnError

    symbols = defaultvariables(SymbolTable())
    perimod._context = Context('<file>', symbols, context, [None, None, None, None, None])
    perimod._symbols = symbols
    perimod._position = TypeObj().setpos(None, None).setcontext(perimod._context)
    perimod._file = 'periscope'
    global periscope
    from peridot.version.modules import periscope



def pprint(format, style=None, color_depth=None):
    print_formatted_text(HTML(format), style=style, color_depth=color_depth)



def test(scripts):
    starttime = datetime.now()



    tests = []
    for i in scripts:
        symbols = defaultvariables(SymbolTable())
        result, exec_context, error = run(i[1], i[0], symbols)

        symbols = {key:value for (key, value) in symbols.symbols.items() if key.startswith('pscope_')}

        if error:
            if isinstance(error, Exc_ReturnError):
                if isinstance(error.returnvalue, periscope.PeriSkip):
                    tests.append((i[1], symbols, error.returnvalue))
                else:
                    tests.append((i[1], symbols, error))
            else:
                tests.append((i[1], symbols, error))
        else:
            tests.append((i[1], symbols))

    totaltests = sum([len(i[1]) for i in tests])

    style = pStyle.from_dict({
        'label'      : 'ansiblue bold',
        'percborder' : 'ansiyellow',
        'percentage' : 'ansiyellow bold',
        'current'    : 'ansiyellow bold',
        'total'      : 'ansiyellow bold',
        'totaltime'  : 'ansiyellow bold',
        'bar'        : 'ansigreen',
    })



    size = get_terminal_size()
    text = ' Testing Started '
    left = '═' * (int(size[0] / 2) - int(len(text) / 2))
    right = '═' * (size[0] - len(text) - len(left))
    pprint(f'\n{escape(left)}<bold>{escape(text)}</bold>{escape(right)}\n', style=style)

    formatters = [
        Text(' '),
        Label(),
        Text(': '),
        Text('[', style='class:percborder'),
        Percentage(),
        Text('][', style='class:percborder'),
        Progress(),
        Text('] ', style='class:percborder'),
        Bar(sym_a='#', sym_b='#', sym_c='-'),
        #Text(' eta [', style='class:percborder'),
        #TimeLeft(),
        #Text('] ', style='class:percborder')
        Text(' ')
    ]

    overview = []
    summary  = []
    errors   = []
    failed = 0
    synfailed = False
    passed = 0

    with ProgressBar(style=style, formatters=formatters, title=HTML(f'<ansigreen>Tracking <bold>{totaltests}</bold> tests...</ansigreen>')) as pb:
        for i in tests:
            if len(i) >= 3:
                if isinstance(i[2], Exc_BaseException):
                    overview.append((i[0], [('fail', i[2])] * len(i[1])))
                    summary .append((i[0], [('fail', i[2].exc, i[2].msg)]))
                    errors  .append(i[2])
                    failed += len(i[1])
                    synfailed = True
                else:
                    overview.append((i[0], [('skip',)] * len(i[1])))
                    summary .append((i[0], [('skip', i[2].msg, i[2].msg)]))
            else:
                overview.append((i[0], []))
                summary .append((i[0], []))
                for key, value in pb(i[1].items(), label=i[0]):
                    res = RTResult()
                    res.register(
                        value.call(key, {}, {}, {})
                    )

                    if isinstance(res.testreturn, periscope.PeriSkip):
                        overview[-1][1].append(('skip',))
                        summary [-1][1].append(('skip', res.testreturn.msg))
                    elif res.error:
                        overview[-1][1].append(('fail', res.error))
                        summary [-1][1].append(('fail', res.error.exc, res.error.msg))
                        errors         .append(res.error)
                        failed += 1
                    else:
                        overview[-1][1].append(('pass',))
                        passed += 1



    size = get_terminal_size()
    text = ' Overview '
    left = '═' * (int(size[0] / 2) - int(len(text) / 2))
    right = '═' * (size[0] - len(text) - len(left))
    pprint(f'\n{escape(left)}<bold>{escape(text)}</bold>{escape(right)}\n', style=style, color_depth=ColorDepth.TRUE_COLOR)

    maxlen = 0
    for i in overview:
        if len(i[0]) > maxlen:
            maxlen = len(i[0])

    for i in overview:
        iover = ''
        for j in i[1]:
            if j[0] == 'pass':
                iover += '<ansigreen>•</ansigreen>'
            elif j[0] == 'skip':
                iover += '<ansiyellow>⏵</ansiyellow>'
            else:
                iover += '<ansired><bold>×</bold></ansired>'
        pprint(f' <ansiblue><bold>{escape(i[0].ljust(maxlen))}</bold></ansiblue>: {iover}', style=style, color_depth=ColorDepth.TRUE_COLOR)



    if failed >= 1 or synfailed:
        size = get_terminal_size()
        text = ' Errors '
        left = '═' * (int(size[0] / 2) - int(len(text) / 2))
        right = '═' * (size[0] - len(text) - len(left))
        pprint(f'\n{escape(left)}<bold>{escape(text)}</bold>{escape(right)}\n', style=style, color_depth=ColorDepth.TRUE_COLOR)

        for i in range(len(errors)):
            err = errors[i].asstring().split('\n')
            err = err[1:-2]
            err = '\n'.join(err)
            print(err)

            if i < len(errors) - 1:
                pprint(f'{escape("─" * get_terminal_size()[0])}', color_depth=ColorDepth.TRUE_COLOR)



    if max([len(i[1]) for i in summary]) >= 1:
        size = get_terminal_size()
        text = ' Summary '
        left = '═' * (int(size[0] / 2) - int(len(text) / 2))
        right = '═' * (size[0] - len(text) - len(left))
        pprint(f'\n{escape(left)}<bold>{escape(text)}</bold>{escape(right)}\n', style=style, color_depth=ColorDepth.TRUE_COLOR)

        maxlen = 0
        for i in summary:
            for j in i[1]:
                if j[0] == 'fail':
                    if len(j[1]) >= 1:
                        maxlen = len(j[1]) + 1

        for i in summary:
            if len(i[1]) >= 1:
                pprint(f' <ansiblue><bold>{escape(i[0])}</bold></ansiblue>:')
                for j in i[1]:
                    if j[0] == 'skip':
                        pprint(f'   <ansiyellow>Skipped</ansiyellow>: {escape(" " * maxlen)}   <ansiyellow><bold>{escape(j[1])}</bold></ansiyellow>')
                    elif j[0] == 'fail':
                        pprint(f'    <ansired>Failed</ansired>: <ansired>{escape(j[1].ljust(maxlen))}</ansired> - <ansired><bold>{escape(j[2])}</bold></ansired>')



    endtime = datetime.now()
    timetaken = endtime - starttime
    timetaken = round(timetaken.total_seconds(), 2)
    timetaken = format(timetaken, '.2f') + 's'


    size = get_terminal_size()
    passedtext = ''
    if passed >= 1 or failed == 0:
        passedtext = f'{passed} passed'
    combiner = ''
    if passed >= 1 and (failed >= 1 or synfailed):
        combiner = ', '
    failedtext = ''
    if failed >= 1 or synfailed:
        failedtext = f'{failed} failed'
    text = f' {passedtext}{combiner}{failedtext} in {timetaken} '
    left = '═' * (int(size[0] / 2) - int(len(text) / 2))
    right = '═' * (size[0] - len(text) - len(left))
    if passed >= 1 and failed == 0 and not synfailed:
        passedtext = f'<bold>{passedtext}</bold>'

    text = f' <ansigreen>{passedtext}</ansigreen>{combiner}<ansired><bold>{failedtext}</bold></ansired> '
    linecolour = 'green'
    if failed >= 1 or synfailed:
        linecolour = 'red'
    pprint(f'\n<ansi{escape(linecolour)}>{left}</ansi{linecolour}>{text}<ansi{linecolour}>in {timetaken} {right}</ansi{linecolour}>\n', style=style, color_depth=ColorDepth.TRUE_COLOR)

    return(bool(failed))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['peridot_periscope', 'peridot_periscope.version']

package_data = \
{'': ['*']}

install_requires = \
['peridot>=1.1.1,<2.0.0']

entry_points = \
{'console_scripts': ['periscope = peridot_periscope:main']}

setup_kwargs = {
    'name': 'peridot-periscope',
    'version': '1.0.0',
    'description': 'Unit testing framework for the Peri.dot Programming Language',
    'long_description': "# Periscope (1.1)\n\n![Periscope Logo](https://raw.githubusercontent.com/toto-bird/Periscope/master/logo.png)\n\n### Usage:\nPeri.dot\n```\nvar periscope = include('periscope')\n\nvar pscope_add() -> Null {\n    return(periscope.skip(\n        'Out of date'\n    ))\n}\n\nvar pscope_sub() -> Null {\n    assert(10 - 5 == 5)\n\n    return(Null)\n}\n\nvar pscope_mul() -> Null {\n    assert(10 - 1 == 10, 'Hmm. Somethings not right')\n\n    return(Null)\n}\n```\n\nBash\n```bash\n# Test files\nperiscope peritests/*\n\n# More help\nperiscope -h\n```\n",
    'author': 'Totobird',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



VERSION = '1.0.0'

if __name__ == "__main__":
    import peridot.version.catch as catch

    catch.location(f'Periscope {VERSION}')
    @catch.catch
    def improvederrormessage():
        ##########################################
        # DEPENDENCIES                           #
        ##########################################

        import sys

        import click
        from   colorama                    import init, Fore, Style
        from   pathlib                     import Path
        init()

        from peridot.__main__              import lang, MODULEVERSION
        import peridot.version.catch       as catch
        import peridot.version.constants   as constants
        import peridot.version.context     as context
        import peridot.version.default     as default
        import peridot.version.exceptions  as exceptions
        import peridot.version.interpreter as interpreter
        import peridot.version.lexer       as lexer
        import peridot.version.nodes       as nodes
        import peridot.version.parser      as parser
        import peridot.version.run         as run
        import peridot.version.tokens      as tokens
        import peridot.version.types       as types
        import perimod

        default._defaultinit(MODULEVERSION, str(Path(__file__).parent), types, context)
        exceptions._exceptionsinit(lang)
        interpreter._interpreterinit(lang, tokens, context, default, constants, types, exceptions, run, perimod)
        lexer._lexerinit(lang, constants, tokens, exceptions)
        nodes._nodesinit(types, tokens)
        parser._parserinit(lang, tokens, exceptions, constants, nodes)
        perimod._perimodinit(catch, types, interpreter, exceptions)
        run._runinit(lexer, parser, context, interpreter)
        types._typesinit(catch, exceptions, context, constants, tokens, nodes, interpreter)

        from   .version                    import test

        test._testinit(run, interpreter, perimod, default, context, types, exceptions)

        ##########################################
        # LOGO                                   #
        ##########################################

        def logo() -> str:
            logolines = [
                f'    _____----_____',
                f'   / \   PERI   / \   Periscope - {VERSION}',
                f' _/  /\  ____  /\  \_  © 2020 Totobird Creations',
                f'/___/ / / __ \/  \___\ ',
                f'\‾‾‾\  /\ ‾‾ / / /‾‾‾/ ',
                f' ‾\  \/  ‾‾‾‾  \/  /‾  https://github.com/toto-bird/Peri.dot',
                f'   \ /  SC()PE  \ /',
                f'    ‾‾‾‾‾----‾‾‾‾‾'
            ]
            logolength = 0
            for i in range(len(logolines)):
                j = logolines[i]
                if len(j) > logolength:
                    logolength = len(j)
            for i in range(len(logolines)):
                logolines[i] = f' ║ {logolines[i].ljust(logolength, " ")} ║'
            logolines.insert(0, f' ╔{"═" * (logolength + 2)}╗')
            logolines.append(f' ╚{"═" * (logolength + 2)}╝')
            logo = '\n'.join(logolines)
            logo = f'{Fore.CYAN}{Style.BRIGHT}{logo}{Style.RESET_ALL}'
            print(logo, end='\n')
            return(logo)

        ##########################################
        # ARGUMENTS                              #
        ##########################################

        @click.command()
        @click.option('-h', '--help',    is_flag=True)
        @click.option('-v', '--version', is_flag=True)
        @click.argument('files', nargs=-1)
        def main(help, version, files):
            if version:
                logo()

            if help:
                print(f'''{Fore.MAGENTA}Usage{Style.RESET_ALL}: {Fore.MAGENTA}{Style.BRIGHT}{Path(sys.argv[0]).name} [OPTIONS]* [FILE]? [ARGS]*{Style.RESET_ALL}

{Fore.BLUE}{Style.BRIGHT}Options:{Style.RESET_ALL}
  {Fore.CYAN}{Style.BRIGHT}-h{Style.RESET_ALL}, {Fore.CYAN}{Style.BRIGHT}--help{Style.RESET_ALL}    - {Fore.CYAN}Display this help message.{Style.RESET_ALL}
  {Fore.CYAN}{Style.BRIGHT}-v{Style.RESET_ALL}, {Fore.CYAN}{Style.BRIGHT}--version{Style.RESET_ALL} - {Fore.CYAN}Display logo and version.{Style.RESET_ALL}''')


            if files:
                scripts = []
                
                for i in files:
                    try:
                        with open(i, 'r') as f:
                            script = f.read()
                            scripts.append(('\n'.join(
                                [
                                    i.lstrip(' ').rstrip(' ').lstrip('\t').rstrip('\t') for i in script.split('\n')
                                ]
                            ), i))

                    except Exception as e:
                        exc = sys.exc_info()
                        print(exceptions.Cmd_CmdArgumentError(f'{exc[0].__name__}: {str(e)}', 'filename', i).asstring())
                        exit(1)

                result = test.test(scripts)

                if result:
                    exit(1)
                else:
                    exit(0)

        main()